#include <click/config.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/glue.hh>

#include <clicknet/ip.h>
#include "aatable.hh"
#include <click/straccum.hh>
#include <clicknet/udp.h>
#include <click/packet_anno.hh>


CLICK_DECLS

AntARPTable :: AntARPTable ()
{
  //Fyller GW tabell med IPadr 0.0.0.0
  for (int ptr=0; ptr<4; ptr++)
  {
    GWtabel[ptr]=IPAddress();
  }


}

AntARPTable :: ~AntARPTable (){}



int AntARPTable::configure(Vector<String> &conf, ErrorHandler *errh)
{
  int res = cp_va_parse(conf, this, errh, cpIPAddress, "local IP 1", &vlan3, cpIPAddress, "local IP 2", &vlan4,
                        cpIPAddress, "local IP 3", &vlan5, cpIPAddress, "local IP 4", &vlan6, cpEnd);
  // legger inn lokale ethernettportadresser
  vlanTabel[0] =vlan3;
  vlanTabel[1] =vlan4;
  vlanTabel[2] =vlan5;
  vlanTabel[3] =vlan6;
  return res;
}


void * AntARPTable::cast(const char *n)
{
  if (strcmp(n, "AARPTable") == 0)
    return (AntARPTable *) this;
  else
    return 0;
}
int AntARPTable::initialize(ErrorHandler *)
{
  return 0;
}


//Her er metoder for hente verdier ut av GWtabellen

//Gir tilbake hvilken gateway tilknyttet remote IPadressetil dette grensesnittet
int AntARPTable :: getGW(in_addr GWadr)
{

  for (int ptr=0; ptr<4; ptr++)
  {
    if( GWtabel[ptr]==GWadr)
    {
      return ptr;
    }
  }
  return -1;
}

// Gir tilbake Ipadresse knyttet til GW
in_addr AntARPTable :: getIP(int ptr){return GWtabel[ptr];}

//Setter IP adresse på en GW
void AntARPTable :: setIP(int ptr, in_addr GWadr)
{
  GWtabel[ptr]=GWadr;
}

// Gir tilbake IPadresse på aktuellt lokalt vlan
in_addr AntARPTable::getVlanIP(int vlanPtr)
{

  return vlanTabel[vlanPtr];
}

// gir -1 tilbake om adressen befinner seg på host ellers 0

int AntARPTable::isThisLocal(in_addr src)
{
  int i=0;
  int test=0;
  while(i<4)
  {
    if (vlanTabel[i]==src)
    {
      test= -1;
    }
    i++;
  }
  return test;
}
//ny sti alle ant
void AntARPTable::addPVEntry(in_addr source, double A, double B, double gamma, int M)
{

  PEntry e = PEntry(source,A, B, gamma, M);
  pathVector.push_back(e);

}
//ny sti eliteant
void AntARPTable::addPVEntryElite(in_addr source, double A, double B, double gamma, int M)
{
  PEntry e = PEntry(source,A, B, gamma, M);
  pathVectorElite.push_back(e);
}

int AntARPTable::chekIfPathExist(in_addr source)
{
  int i=0;
  while (i<pathVector.size())
  {
    if (IPAddress(pathVector.at(i).src) == IPAddress(source))
    {
      return i;
    }
    i++;
  }
  return -1;
}
//oppdatere sti
void AntARPTable::updatePathEntry(int entry, double _A, double _B, double _gamma, int _M)
{
  pathVector.at(entry).A=_A;
  pathVector.at(entry).B=_B;
  pathVector.at(entry).gamma=_gamma;
  pathVector.at(entry).M=_M;
}
//oppdatere sti eliteant
void AntARPTable::updatePathEntryElite(int entry, double _A, double _B, double _gamma, int _M)
{
  pathVectorElite.at(entry).A=_A;
  pathVectorElite.at(entry).B=_B;
  pathVectorElite.at(entry).gamma=_gamma;
  pathVectorElite.at(entry).M=_M;
}


EXPORT_ELEMENT(AntARPTable)
#include <click/vector.cc>
CLICK_ENDDECLS
