#ifndef ANTARPTABLE_HH
#define ANTARPTABLE_HH
#include <click/element.hh>
#include <click/ipaddress.hh>
#include <click/vector.hh>

CLICK_DECLS



class AntARPTable : public Element
{
public:
  AntARPTable();
  ~AntARPTable();
  const char *class_name() const { return "AntARPTable"; }
  void *cast(const char *);
  int configure(Vector<String>&, ErrorHandler*);
  int initialize(ErrorHandler *);

  //path entry
  struct PEntry
  {
    PEntry(in_addr _src, double _A, double _B, double _gamma, int _M) : src(_src), A(_A), B(_B), gamma(_gamma), M(_M) {}
    PEntry() {}
    in_addr src;
    double A, B, gamma;
    int M;
  };
  //legger til en ny entry til normalvektor
  void addPVEntry(in_addr source, double A, double B, double gamma, int M );
  //legger til en ny entry til elitevektor
  void addPVEntryElite(in_addr source, double A, double B, double gamma, int M );
  // sjekker om stien er der og returnerer entrynummer
  int chekIfPathExist(in_addr);
  //oppdaterer eksisterende sti med nye verdier
  void updatePathEntry(int entry, double A, double B, double gamma, int M);
  //oppdaterer eksisterende elitesti med nye verdier
  void updatePathEntryElite(int entry, double A, double B, double gamma, int M);

  int getGW(in_addr);  //Gir tilbake hvilken GW denne IP adresse befinner seg på
  in_addr getIP(int); //Gir tilbake en IP avhengig av GW
  void setIP(int, in_addr); // Setter IP adresse på en GW
  in_addr getVlanIP(int); // Hvilken IP adresse har vlan
  int isThisLocal(in_addr dest); // sjekk om denne IP adressen er lokal
  Vector<AntARPTable::PEntry> pathVector;  // Vektor over alle ant sortert på source
  Vector<AntARPTable::PEntry> pathVectorElite; //Vektor over alle eliteant sortert på source
private:
  int GWpointer;
  int interface;
  in_addr vlanTabel[4]; //tabell over lokale IPadresser
  in_addr vlan3, vlan4, vlan5, vlan6;
  in_addr GWtabel[4]; //tabell over hvilke IPadresser som er på andre siden av linken

};


CLICK_ENDDECLS
#endif
