/*
 * anttable.{cc,hh} -- a routing table for CE Ant rounting
 * Ingebrigt Fuglem
 */

#include <click/config.h>
#include <click/error.hh>
#include <click/confparse.hh>
#include "anttable.hh"
CLICK_DECLS

AntTable::AntTable() 
{
  last_ptr = -1;
}

AntTable::~AntTable()
{
}

void *
AntTable::cast(const char *n)
{
  if (strcmp(n, "AntTable") == 0)
    return (AntTable *) this;
  else
    return 0;
}

int
AntTable::configure(Vector<String> &conf, ErrorHandler *errh)
{
	int before = errh->nerrors();
  for (int i = 0; i < conf.size(); i++) {
  	Vector<String> words;
	  cp_spacevec(conf[i], words);

		IPAddress address;
    IPAddress netmask;
    int physical_port;
		int cost;
		bool ok = false;
		//ok = cp_ip_address(words[0], &interface, this);
    ok = cp_ip_prefix(words[0], &address, &netmask, true, this);
		if (words.size() == 3) {
			ok = cp_integer(words[1], &physical_port) & cp_integer(words[2], &cost);
		}
		if (ok) {
			click_chatter("Adding interface %s and cost %d on physical port %d to table of interfaces", address.unparse_with_mask(netmask).cc(), cost, physical_port);
			_interfaces.push_back(IfEntry(address, netmask, physical_port, cost));
		} else {
			errh->error("argument %d should be `ADDR COST'", i+1);
		}
	}
	return (errh->nerrors() != before ? -1 : 0);
}

int
AntTable::initialize(ErrorHandler *)
{
  return 0;
}

bool
AntTable::is_host(const IPAddress &ip)
{
  for (int i = 0; i < _interfaces.size(); i++) {
    if (_interfaces[i].address == ip) {
      return true;
    }
  }
  return false;
}

int 
AntTable::get_first(const IPAddress &_src_ip, const IPAddress &_dest_ip)
{
  last_ptr = -1;
  last_src_ip = _src_ip;
  last_dest_ip = _dest_ip;
  
  for (int i = 0; i < _ant_routes.size(); i++) {
    if (_ant_routes[i].src_ip == _src_ip && _ant_routes[i].dest_ip == _dest_ip)
    {
      last_ptr = i;
      return i;
    }
  }
  if (last_ptr < 0) {
    // No routing table exists for this src+dst. Add all interfaces to _ant_routes
    for (int i = 0; i < _interfaces.size(); i++) {
      const IfEntry if_entry = _interfaces[i];
      click_chatter("adding src %s, dest %s, interface %s, port %d, physical_port %d, cost %d to Ant routing table", 
        _src_ip.s().c_str(), _dest_ip.s().c_str(), if_entry.address.s().c_str(), i, if_entry.physical_port, if_entry.cost);
      ATEntry e = ATEntry(_src_ip, _dest_ip, if_entry.address, i, if_entry.physical_port, if_entry.cost, 0, 0, 0, 0);

      //Add ATEntry to _ant_routes table
      _ant_routes.push_back(e);
    }
    last_ptr = 0;
  }
  return last_ptr;
}

int
AntTable::get_next(int previous)
{
  if (previous < 0 || previous == _ant_routes.size()) 
  {
    return -1;
  }
    
  for (int i = previous+1; i < _ant_routes.size(); i++) {
    if (_ant_routes[i].src_ip == last_src_ip && _ant_routes[i].dest_ip == last_dest_ip)
    {
      last_ptr = i;
      return i;
    }
  }
  last_ptr = -1;
  return -1;
}

EXPORT_ELEMENT(AntTable)
//ELEMENT_HEADER("./package/anttable.hh")
#include <click/vector.cc>
CLICK_ENDDECLS
