
#ifndef ANTTABLE_HH
#define ANTTABLE_HH
#include <click/element.hh>
#include <click/ipaddresslist.hh>
#include <click/vector.hh>

CLICK_DECLS

// Ant routing table.
// Lookup by longest prefix.
// Each entry contains a gateway and an output index.

class AntTable : public Element { 
public:
  
	AntTable();
	~AntTable();

  const char *class_name() const		{ return "AntTable"; }
  void *cast(const char *);
  const char *processing() const		{ return "h/h"; }
  const char *flow_code() const                 { return "x/y"; }
  
  int configure(Vector<String> &, ErrorHandler *);
  int initialize(ErrorHandler *);

private:
  IPAddress last_src_ip, last_dest_ip;
  
public:
  int last_ptr;
	
	/* Routing table entry */
	struct ATEntry {
		ATEntry(IPAddress _src_ip, IPAddress _dest_ip, IPAddress _interface, int _port, 
      int _physical_port, int _cost, int _pheromone, double _a, double _b, double _c ) :
			src_ip(_src_ip), dest_ip(_dest_ip), interface(_interface), if_carrier_ok(true), 
      port(_port), physical_port(_physical_port), cost(_cost), pheromone(_pheromone), a(_a), b(_b), c(_c)
		{}
    ATEntry(const ATEntry &_entry) : 
      src_ip(_entry.src_ip), dest_ip(_entry.dest_ip), interface(_entry.interface), if_carrier_ok(true), 
      port(_entry.port), physical_port(_entry.physical_port), cost(_entry.cost), pheromone(_entry.pheromone), 
      a(_entry.a), b(_entry.b), c(_entry.c)
    {}
		ATEntry() {}
		IPAddress src_ip;
		IPAddress dest_ip;
		IPAddress interface;
    bool if_carrier_ok;
    int port;
    int physical_port;
		int cost;
		double pheromone;
		double a, b, c;
	};
	
  // Interfaces with cost 
	struct IfEntry {
		IPAddress address;
    IPAddress netmask;
    int physical_port;
		int32_t cost;
    double rate;
		IfEntry(IPAddress address, IPAddress _netmask, int _physical_port, int _cost) : 
      address(address), netmask(_netmask), physical_port(_physical_port), cost(_cost), rate(0) {}
		IfEntry() : physical_port(0), cost(0), rate(0) {}
	};

	typedef Vector<AntTable::ATEntry> ATTable;
	typedef Vector<AntTable::IfEntry> IfTable;
	
	IfTable _interfaces; 	/* All interfaces on this router */
	ATTable _ant_routes;  /* the route table */
	
  bool is_host(const IPAddress &ip);
  int get_first(const IPAddress &_src_ip, const IPAddress &_dest_ip);
  int get_next(int previous);
//	void get_entries_for_src_dest(Vector<ATEntry> &art, const IPAddress &_src_ip, const IPAddress &_dest_ip);
	void clear() { _ant_routes.clear(); }
	int getNumberOfInterfaces() { return _interfaces.size(); }
  IfEntry getInterface(int index) { assert(index<_interfaces.size()); return _interfaces[index]; }
};

CLICK_ENDDECLS
#endif
