/*
 * backwardant.{cc,hh} -- element looks up next-hop address in
 * Ant routing table
 * Ingebrigt Fuglem
 *
*/

#include <click/config.h>
#include "backwardant.hh"
#include <click/ipaddress.hh>
#include <clicknet/ip.h>
#include <clicknet/udp.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/glue.hh>
#include <click/straccum.hh>
#include "anttable.hh"
CLICK_DECLS

BackwardAnt::BackwardAnt() : Element(1, 1), _antTable(0)
{
}

BackwardAnt::~BackwardAnt()
{
}

int
BackwardAnt::configure(Vector<String> &conf, ErrorHandler *errh)
{
  int res = cp_va_parse(conf, this, errh,
      cpElement, "AntTable element", &_antTable,
      cpEnd);
//  int ifs = _antTable->getNumberOfInterfaces();
//  click_chatter("BackwardAnt configured with %d interfaces", ifs);
//  set_noutputs(ifs);
  return res;
}

int
BackwardAnt::initialize(ErrorHandler *errh)
{
  if(_antTable == 0){
    return errh->error("no AntTable argument");
  }

  return 0;
}

in_addr
BackwardAnt::getDestination(Packet *p)
{
  const uint8_t *oa = p->network_header();
  int hlen = p->network_header_length();
  struct in_addr dest;

  // Walk through options header
  for (int oi = sizeof(click_ip); oi < hlen; ) {
    unsigned type = oa[oi];
    int xlen = oa[oi + 1];  // length
    if (type == IPOPT_EOL) {
      /* end of option list */
      break;
    } else if (type == IPOPT_LSRR || type == IPOPT_SSRR) {
      int po = oa[oi+2]; // pointer
      memcpy(&dest, oa + oi + po - 1, 4);   // Next hop
      if (dest.s_addr == 0) {
        // Next hop is actual destination
        return p->ip_header()->ip_dst;
      } else {
        // Find first 0.0.0.0 address
        while (dest.s_addr != 0 && po + 3 < xlen) {
          po += 4;
          memcpy(&dest, oa + oi + po - 1, 4);
//click_chatter("po=%d, dest=%x", po, dest.s_addr);
        }
        po -= 4;    // Move up
        memcpy(&dest, oa + oi + po - 1, 4);
//click_chatter("po=%d, dest=%x", po, dest.s_addr);
        return dest;
      }
    }
    oi += xlen;
  }
  return p->ip_header()->ip_dst;
}

double atod(const char *pch) {
  int i = 0;
  cp_real10(String(pch), 5, &i);
  double d = ((double)i)/100000;
  //click_chatter("in: %s, i: %d out: %f", pch, i, d);
  return d;
}

void
BackwardAnt::readTempAndCost(Packet *p, int *explore, double *tmpE, double *cost, double *temperature, double *beta) {
  int oi = p->network_header_length() + sizeof(struct click_udp);   // Point to start of UDP payload
  uint16_t len = p->length() - p->network_header_length();  // entire ip packet - ip header
  char payload[len];
  strncpy(payload, (char *)p->data() + oi, len);
   
//  char *payload = (char *)p->data() + oi;
  
  // oa = explore/tmpE/cost/temp/beta/... = "0/1.23440.54/1.0023/2.666/..."

char * realPayload=strtok( payload,"!" );

  if (payload != NULL){


 
  char *pch = strtok(realPayload,"/");
  if (pch != NULL)
  {
    cp_integer(String(pch), explore);
    pch = strtok (NULL, "/");
    if (pch != NULL)
    {
      *tmpE = atod(pch);
      pch = strtok (NULL, "/");
      if (pch != NULL)
      {
        *cost = atod(pch);
        pch = strtok (NULL, "/");
        if (pch != NULL)
        {
          *temperature = atod(pch);
          pch = strtok (NULL, "/");
          if (pch != NULL)
          {
            *beta = atod(pch);
          }
        }
      }
    }
  }
}
}
Packet *
BackwardAnt::simple_action(Packet *p)
{
  assert(_antTable);

  double beta = 0;    // history parameter
  double T = 0;       // cost
  double tmpga0 = 0;  // temperature
  double tmpA=0;
  double tmpB=0;
  double tmpC=0;
  double tmpE = 0;
  int explore=0;
  _state = 0;
  // Get source and destination
  const IPAddress src_ip = IPAddress(p->ip_header()->ip_src);
  // Get actual destination from packet
  const IPAddress dest_ip = IPAddress(getDestination(p));

  // Get temperature and cost from UDP payload
  readTempAndCost(p, &explore, &tmpE, &T, &tmpga0, &beta);
  click_chatter("BACKWARD ant #%d: %s => %s with tmpE=%4.4f, T=%f, tmpga0=%f, beta=%f", 
     ntohs(p->ip_header()->ip_id), src_ip.s().c_str(), dest_ip.s().c_str(), tmpE, T, tmpga0, beta);

  if (explore != 0) {
    _state = 101;
    click_chatter("No update - pass through");
    return (p);
  }
  _state = 1;
  
  int i = _antTable->get_first(dest_ip, src_ip);

  if (i < 0) {
    // No routes found - now what?? @todo
    click_chatter("Error: No routes found");
    _state = 102;
    return 0;
  }

  const IPAddress destination = IPAddress(p->ip_header()->ip_dst);
  click_chatter("Incomming interface: %s", destination.s().cc());
  _state = 2;
  // For each interface, sum up pheromone 
  while (i >= 0) {
    click_chatter("i=%d",i);
    _state++;
    AntTable::ATEntry *at_entry = &_antTable->_ant_routes[i];
    tmpA = at_entry->a;
    tmpB = at_entry->b;
    tmpC = at_entry->c;
    //    click_chatter("IF[%d]: tmpA=%f, tmpB=%f, tmpC=%f, tmpE=%f, T=%f, tmpga0=%f", at_entry->port, tmpA, tmpB, tmpC, tmpE, T, tmpga0);    
//    AntTable::IfEntry interface = _antTable->getInterface(i);
//click_chatter("interface: %s/%s destionation: %s/%s", interface.address.s().cc(), interface.netmask.s().cc(), destination.s().cc(), interface.netmask.s().cc());
    if (at_entry->interface.addr() == destination.addr()) {
      // This is the selected path
      tmpA = tmpA + tmpE*(1 + T/tmpga0*(1+T/(2*tmpga0)));
      tmpB = tmpB + tmpE*(T + T*T/tmpga0);
      tmpC = tmpC + tmpE*T*T/2;
      //      click_chatter("IF[%d]: tmpA=%4.4f, tmpB=%4.4f, tmpC=%4.4f, tmpE=%4.4f, T=%f, tmpga0=%f, beta=%f (updated)", at_entry->port, tmpA, tmpB, tmpC, tmpE, T, tmpga0, beta);    
    }
    
    // Update a, b and c
    at_entry->a = tmpA*beta;
    at_entry->b = tmpB*beta;
    at_entry->c = tmpC*beta;
    double tmp = tmpA;
    
    if (tmpB * tmpC * tmpga0 > 0) {
      if (1/tmpga0 < tmpB/(2*tmpC)) {
        tmp = tmp + tmpC/(tmpga0*tmpga0) - tmpB/tmpga0;
      } else {
        tmp = tmp - (tmpB*tmpB)/(4*tmpC);
      }
    }
    // And finally, update pheromone
    if (at_entry->interface == IPAddress(p->ip_header()->ip_dst)) {
      // This is the selected path
      at_entry->pheromone = tmpE + tmp;
    } else {
      at_entry->pheromone = tmp;
    }
    click_chatter("IF[%s]: pher=%f (a=%f, b=%f, c=%f)", at_entry->interface.s().c_str(), 
		  at_entry->pheromone, at_entry->a, at_entry->b, at_entry->c);
    i = _antTable->get_next(i);
  }
  _state += 1000;
  //  click_chatter("%s", routingTable());
  return(p);
}

String
BackwardAnt::routingTable() {
  StringAccum retval;
  retval << "NO |      SRC        |      DST        |      IF         |  Pher  |   A    |   B    |   C\n";
         //  168.24.1.10     | 129.241.219.168 | 129.241.219.201 |  4,768 |  2,721 |  3,212 |  0,283
  for (int i = 0; i<_antTable->_ant_routes.size(); i++) {
    AntTable::ATEntry *at_entry = &_antTable->_ant_routes[i];
    char line[100];
    sprintf(line, " %d | %15s | %15s | %15s | %2d.%03d | %2d.%03d | %2d.%03d | %2d.%03d ", i, 
      at_entry->src_ip.s().c_str(), at_entry->dest_ip.s().c_str(), at_entry->interface.s().c_str(),
      (int)(at_entry->pheromone*1000)/1000, (int)(at_entry->pheromone*1000)-(int)at_entry->pheromone*1000,
      (int)(at_entry->a*1000)/1000, (int)(at_entry->a*1000)-(int)at_entry->a*1000,
      (int)(at_entry->b*1000)/1000, (int)(at_entry->b*1000)-(int)at_entry->b*1000,
      (int)(at_entry->c*1000)/1000, (int)(at_entry->c*1000)-(int)at_entry->c*1000);
    retval << line << "\n";
  }
  return retval.take_string();
}

enum { H_STATE, H_ROUTE_TABLE };
       
String
BackwardAnt::read_handler(Element *e, void *thunk)
{
  BackwardAnt *me = (BackwardAnt *)e;
  switch ((intptr_t)thunk) {
   case H_STATE:
    return String(me->_state) + "\n";
   case H_ROUTE_TABLE:
    return me->routingTable();
   default:
    return "<error>\n";
  }
}

void
BackwardAnt::add_handlers()
{
//  add_read_handler("routing", BackwardAnt_routingTable, 0);
  add_read_handler("routing", read_handler, (void *)H_ROUTE_TABLE);
  add_read_handler("state", read_handler, (void *)H_STATE);
}


CLICK_ENDDECLS
ELEMENT_REQUIRES(AntTable)
EXPORT_ELEMENT(BackwardAnt)
