#include <click/config.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/glue.hh>
#include <clicknet/ip.h>
#include "aatable.hh"
#include <clicknet/udp.h>
#include <click/packet_anno.hh>
#include <click/straccum.hh>
#include <math.h>
#include "updateaatbl.hh"
#include <click/ipaddress.hh>



CLICK_DECLS
UpdateAATbl::UpdateAATbl() : Element(1, 1) , _aaTable (0)
{
  //legger inn initiellle "null" verdier for noen tabeller
  for (int ptr=0; ptr<4; ptr++)
  {
    vlanTable[ptr]=IPAddress();
    remTable[ptr]=IPAddress();
    lastStatusLink[ptr]=0;
    remTable[ptr]=IPAddress();
  }
  for (int i=0 ; i<6 ; i++)
  {
    if_status[i]=0;
  }
}

UpdateAATbl::~ UpdateAATbl()
{}
int UpdateAATbl ::configure(Vector<String> &conf, ErrorHandler *errh)
{
  int anim_ret=0;
  int res = cp_va_parse(conf, this, errh,  cpElement, "AntARPTable element", &_aaTable, cpEnd);
  animate_return=3;  //Antall animeringspakker som skal returneres pr hendelse

  for (int ptr=0; ptr<4; ptr++)
  {
    linkopptabell[ptr]=anim_ret+1;
    linknedtabell[ptr]=anim_ret+1;
  }

  return res;
}

void UpdateAATbl::notify_noutputs(int n)
{
  set_noutputs(n < 2 ? 1 : n);
}


void UpdateAATbl::oppdater_tabell(Packet *p)
{
  StringAccum adresse; // brukes til printe ut adresse
  in_addr interface=IPAddress();
  in_addr lastRRaddr=IPAddress();
  const uint8_t *oa = p->network_header();
  int hlen = p->network_header_length();
  int oi = sizeof(click_ip);// lengde på Ipheader før options
  int xlen = oa[oi + 1];
  int last_ptr=oa[oi+2];
  int interface_nr=(PAINT_ANNO(p));

  //Sjekker om pointer er større enn minlengden dvs innholder noen RR
  //Hvis det er tilfellet legger vi til siste feltet i route data til rutingGWtabell

  if (last_ptr>4)
  {
    memcpy(&lastRRaddr , oa +oi + last_ptr-5, 4);// henter remote IPadresse som ligger i RR
  }
  else
  {
    lastRRaddr = IPAddress(p->ip_header()->ip_src); // hvis ingen RR er fortatt (ptr=4) er srcadressen som er remote
  }


  if(_aaTable->getIP(interface_nr)!=lastRRaddr)
  {
    int testPtr=_aaTable->getGW(lastRRaddr);
    if (testPtr >=0)
    {
      _aaTable->setIP(testPtr, IPAddress());// Fjerner innslag hvis på andre adresser
    }
    _aaTable->setIP(interface_nr, lastRRaddr); // Oppdater
  }
  for (int ptr=0; ptr<4; ptr++)	// her skal vi sjekke om iterface er oppe hvis så er tilfellet er et et oppslag i tabellen?
  {
    if (if_status[3-ptr]>lastStatusLink[ptr])
    {  // nylikup
      if(_aaTable->getIP(ptr)!=IPAddress())
      {
        linkopptabell[ptr]=0;
        linknedtabell[ptr]=animate_return;
        remTable[ptr]=_aaTable->getIP(ptr);
        lastStatusLink[ptr]=if_status[3-ptr]; // registrer nåværende status i laststaus
      }
    }

    if (if_status[3-ptr]<lastStatusLink[ptr])
    { // linkned
      linknedtabell[ptr]=0;
      linkopptabell[ptr]=animate_return;
      _aaTable->setIP(ptr, IPAddress());
      lastStatusLink[ptr]=if_status[3-ptr]; // registrer nåværende status i laststatus
    }

    if (linkopptabell[ptr]<=animate_return)
    {  //skal sende antall pakker konfigurert i  innverdi
      if (_aaTable->getIP(ptr)!=IPAddress())
      {
        linkopptabell[ptr]++;
        Packet *xp= send_notify(p, _aaTable->getVlanIP(interface_nr) , 7); //lager en pakke for å oppdatere NAM  linkopp;

        Packet *yp= addRemoteAddr(xp,_aaTable->getIP(ptr)); //legger in remoteadressen på slutten av datalasten;
        output(1).push(yp); //sender linkopppakke tilbake til sender;
      }
    }

    if (linknedtabell[ptr]<animate_return)
    {  //skal sende antall pakker konfigurert i  innverdi

      if (remTable[ptr]!=IPAddress())
      {
        linknedtabell[ptr]++;
        Packet *xp= send_notify(p, _aaTable->getVlanIP(interface_nr) , 6); //lager en pakke for å oppdatere NAM at linkned;
        Packet *yp= addRemoteAddr(xp, remTable[ptr]); //legger in remoteadressen på slutten av datalasten;
        output(1).push(yp); //sender linknedpakke tilbake til sender
        if(linknedtabell[ptr]==animate_return)
        {
          remTable[ptr]=IPAddress();
        }
      }
    }

  }

}


Packet * UpdateAATbl::addRemoteAddr(Packet *p, in_addr lastRRaddr)
{
  WritablePacket *wp = 0;
  if (!(wp = p->uniqueify()))
    return 0;
  // Add remote addresse to end of packet
  String remoteAddr="/"+IPAddress(lastRRaddr).unparse ()+"!";
  int udp_end = wp->length();
  wp->put(remoteAddr.length());

  if (!wp)
    return 0;
  memcpy(wp->data() + udp_end, remoteAddr.c_str(), remoteAddr.length());

  // Update IP and UDP length
  click_ip *iph = wp->ip_header();
  click_udp *udph = wp->udp_header();
  uint16_t len = ntohs(udph->uh_ulen)+remoteAddr.length()+2;
  udph->uh_ulen = htons(len);
  iph->ip_len = htons(p->length());

  return update_checksum(wp);
}




/* Check if interface is up */
bool UpdateAATbl::is_if_up(struct in_addr interface)
{
#ifdef HAVE_CLICK_KERNEL
  struct net_device *dev;
  for (dev=dev_base; dev; dev = dev->next)
  {
    struct in_ifaddr *ifa;
    if (dev->ip_ptr != NULL)
    {
      for (ifa = ((struct in_device*)(dev->ip_ptr))->ifa_list;ifa; ifa = ifa->ifa_next)
      {
        if (ifa->ifa_local == interface.s_addr)
        {
          return (netif_carrier_ok(dev) & (dev->flags & IFF_UP));
        }
      }
    }
  }
#else
  return true;
#endif
}



// Linksys stuff
void UpdateAATbl::get_linksys_status2(int status[])
{
  char buffer[1024];
  FILE* fd = popen("robocfg show", "r");
  if (fd == NULL)
  {
    click_chatter("Unable to run robocfg");
  }

  int port = -1;
  while (NULL != fgets(buffer, sizeof(buffer), fd))
  {
    // Port 0(W):  DOWN enabled stp: none vlan: 6 mac: 00:00:00:00:00:00
    char port_status[10];
    if (sscanf(buffer, "%*s%d%*s%s", &port, port_status) == 2)
    {
      if (port<6 && strcmp(port_status, "DOWN") == 0)
      {
        status[port] = 0;
      }
      else
      {
        status[port] = 1;
      }
    }
  }
  pclose (fd);
}




Packet * UpdateAATbl::send_notify(Packet *p, in_addr interface, int _flag)
{

  StringAccum test5;
  WritablePacket *q = p->clone()->uniqueify();
  click_ip *ip = q->ip_header();
  click_udp *udp = q->udp_header();

  // Set src and dst ip-addr and port no
  ip->ip_src = interface;
  ip->ip_dst = p->ip_header()->ip_src;
  ip->ip_id = random();
  ip->ip_ttl = 9;
  udp->uh_sport = p->udp_header()->uh_dport;
  udp->uh_dport = p->udp_header()->uh_sport;

  // Use route record to create source routing
  uint8_t *oa = q->network_header();
  int hlen = q->network_header_length();


  // Walk through options header
  for (int oi = sizeof(click_ip); oi < hlen; )
  {
    unsigned type = oa[oi];
    if (type == IPOPT_EOL)
    {
      /* end of option list */
      break;
    }
    else if (type == IPOPT_RR)
    {
      // Route Record
      oa[oi] = IPOPT_LSRR;
      int last_ptr = oa[oi+2]-5; // points to last route record
      int first_ptr = 3;

      if (last_ptr >= 3)
      {
        // 1 or more
        struct in_addr dest;
        memcpy(&dest, oa + oi + last_ptr, 4);   // Use last as destination
        ip->ip_dst = dest;
        memcpy(oa + oi + last_ptr, &p->ip_header()->ip_src, 4);   // Put source in last position
        last_ptr -= 4;
      }
      while (last_ptr > first_ptr)
      {
        // swap
        struct in_addr temp;
        memcpy(&temp, oa + oi + last_ptr, 4);
        memcpy(oa + oi + last_ptr, oa + oi + first_ptr, 4);
        memcpy(oa + oi + first_ptr, &temp, 4);

        first_ptr += 4;
        last_ptr -= 4;
      }
      oa[oi+2] = 4;
      break;
    }
  }

  int udph = q->network_header_length() + sizeof(struct click_udp);   // Point to start of UDP payload
  String flag = String(_flag);
  memcpy(q->data() + udph, flag.c_str(), flag.length());

  q->set_dst_ip_anno(IPAddress(ip->ip_dst));
  q->set_ip_header(ip, sizeof(click_ip));
  q->set_network_header_length(p->network_header_length());

  return update_checksum(q);
}

Packet * UpdateAATbl::update_checksum(WritablePacket *p)
{
  WritablePacket *wp = 0;
  if (!wp)
  {
    if (!(wp = p->uniqueify()))
      return 0;
    else
    {
      click_ip *iph = wp->ip_header();
      click_udp *udph = wp->udp_header();

      uint16_t len = ntohs(udph->uh_ulen);
      int hlen = p->network_header_length();

      iph->ip_sum = 0;
      iph->ip_sum = click_in_cksum(p->network_header(), hlen);

      udph->uh_sum = 0;
      unsigned csum = click_in_cksum((unsigned char *)udph, len);
      udph->uh_sum = click_in_cksum_pseudohdr(csum, iph, len);
    }
  }

  return (wp);
}


void UpdateAATbl::push(int, Packet *p)
{
  get_linksys_status2(if_status);
  oppdater_tabell(p);
  output(0).push(p);


}
CLICK_ENDDECLS
ELEMENT_REQUIRES(AntARPTable)
EXPORT_ELEMENT(UpdateAATbl)
