

#include <click/config.h>
#include <click/confparse.hh>
#include <click/error.hh>
#include <click/glue.hh>
#include <clicknet/ip.h>
//#include <elements/ip/lineariplookup.hh>
#include <click/vector.hh>
#include "aatable.hh"
#include "updatepathtbl.hh"
#include <clicknet/udp.h>

#include <click/packet_anno.hh>
#include <click/straccum.hh>
#include <math.h>
#include <click/ipaddress.hh>



CLICK_DECLS

UpdatePathTable::UpdatePathTable() : Element(1, 1) , _aaTable (0)
{

  antallfeilGP=0;
  antallfeilCP=0;
  antallfeilGUA=0;


}

UpdatePathTable::~ UpdatePathTable()
{}
int
UpdatePathTable ::configure(Vector<String> &conf, ErrorHandler *errh)
{
  int res = cp_va_parse(conf, this, errh,  cpElement, "AntARPTable" ,&_aaTable, cpEnd);
  return res;
}


void
UpdatePathTable::notify_noutputs(int n)
{
  set_noutputs(n < 2 ? 1 : n);
}

// sletter elementene i vektoren
void
UpdatePathTable::clearVector()
{
  costVector.clear();
}

void
UpdatePathTable::addCostVectorValue(int cost)
{
  costVector.push_back(cost);
}
int
UpdatePathTable::getCostVectorValue(int cost)
{
  return costVector[cost];
}

int
UpdatePathTable::getCostVectorSize()
{
  return costVector.size();
}


Packet *
UpdatePathTable::handle_options(Packet *p)

{
  in_addr dest_adr = IPAddress(p->ip_header()->ip_dst);
  WritablePacket *wp = p->clone()->uniqueify();
  //   if (!(wp = p->uniqueify()))
  //	return 0;

  //  in_addr interface=IPAddress();
  in_addr lsAdr=IPAddress();
  uint8_t *oa = wp->network_header();
  int hlen = wp->network_header_length();
  int oi = sizeof(click_ip);// lengde på Ipheader før options

  int last_ptr=oa[oi+2];
  int interface_nr=(PAINT_ANNO(p));
  lsAdr=_aaTable->getVlanIP(interface_nr);

  if (dest_adr!=lsAdr)
  {
    if ( last_ptr<40)
    {
      memcpy( oa +oi +last_ptr-1, &lsAdr , 4);//koperer
      oa[oi+2] +=4;
      out=1;
    }
  }
  if (wp)
  {
    click_ip *iph = wp->ip_header();
    iph->ip_sum = 0;
    iph->ip_sum = click_in_cksum(p->network_header(), hlen);
  }

  return (wp ? wp : p);










}
void
UpdatePathTable::getPayload(Packet *p,  int *explore, double *rho, double *beta, int *totCost)
{
  try
  {
    double dummy=0;
    int hopCounter;
    *totCost=0;
    int kost=0;
    int xCost=0;
    clearVector();  // sletter elementene i vektoren

    int oi = p->network_header_length() + sizeof(struct click_udp);   // Point to start of UDP payload
    uint16_t len = p->length() - p->network_header_length()-8;  // entire ip packet - ip header
    char payload[len];
    strncpy(payload, (char *)p->data() + oi, len);

    char *pch = strtok(payload,"/");
    if (pch != NULL)
    {
      *explore = atoi(pch);
      if (*explore==0||*explore==1)
      {

        pch = strtok (NULL, "/");

        hopCounter=0;
        if (pch != NULL)
        {
          while (pch != NULL)
          {
            xCost=atoi(pch);
            addCostVectorValue(xCost);
            *totCost=*totCost+xCost;
            pch = strtok (NULL, "/");
            hopCounter++;
          }
        }
      }

      if (*explore==2) // init exp ant
      {
        pch = strtok (NULL, "/");
        if (pch != NULL)
        {
          *rho = atof(pch);
          pch = strtok (NULL, "/");
          if (pch != NULL)
          {
            *beta = atof(pch);

            pch = strtok (NULL, "/");
            if (pch != NULL)
            {
              dummy = atof(pch);
              pch = strtok (NULL, "/");
              if (pch != NULL)
              {
                dummy = atof(pch);
                pch = strtok (NULL, "/");
                hopCounter=0;

                while (pch != NULL)
                {
                  xCost=atoi(pch);
                  addCostVectorValue(xCost);
                  *totCost=*totCost+xCost;
                  pch = strtok (NULL, "/");
                  hopCounter++;
                }
              }
            }

          }
        }
      }
    }


  }
  catch (...)
  {
    antallfeilGP++;
    feil=1;
  }
}




void UpdatePathTable::calculatePayload(Packet *p, double *X, double *gamma, double *beta, double *rho, double *tmpNAMtime, int *eliteant, int *totCost )


try
{
  double tmpA=0;
  double A=0;
  double tmpB=0;
  double B=0;
  double tmpgamma=0;
  double tmpX=0;
  int tmpM=0;
  int M=0;
  const IPAddress src_adr = IPAddress(p->ip_header()->ip_src);
  int vectorEntAcc=_aaTable->chekIfPathExist(src_adr);


  //tester om vi har abnormale verdier Hvis så er tilfelle legger vi inn nye verdier for at systement ikke skal gå ned
  if (*rho<=0)
  {

    *rho=0.05;
    antallfeilCP++;
  }

  if (*beta<=0)
  {

    *beta=0.93;
    antallfeilCP++;
  }

  // Hvis totCost er feil lest av pakken regnes det ut en verdi utfra eliteantlimit
  if (*totCost<=0)
  {

    antallfeilCP++;
    if (vectorEntAcc>=0)
    {
      *totCost=(_aaTable->pathVector.at(vectorEntAcc).gamma)*(-1)*log(*rho);
    }
    // Hvis det er første ant som svikter legges inn en høy verdi
    else
    {
      *totCost=500;
    }
  }

  if (vectorEntAcc<0)	// vi har en ny sti
  {
    tmpgamma=-1*(*totCost/log(*rho));
    *gamma=tmpgamma;
    *eliteant=0;
  }
  else // exksisterende sti
  {
    tmpM=_aaTable->pathVector.at(vectorEntAcc).M;
    tmpgamma=_aaTable->pathVector.at(vectorEntAcc).gamma;
    tmpA=_aaTable->pathVector.at(vectorEntAcc).A;
    tmpB=_aaTable->pathVector.at(vectorEntAcc).B;

    M=_aaTable->pathVectorElite.at(vectorEntAcc).M;
    *gamma=_aaTable->pathVectorElite.at(vectorEntAcc).gamma;
    A=_aaTable->pathVectorElite.at(vectorEntAcc).A;
    B=_aaTable->pathVectorElite.at(vectorEntAcc).B;
  }

  double eliteAntTest=tmpgamma*(-1)*log(*rho);  //tester om dette er en eliteant
  // regner ut temperaturdata som skal tas vare på
  double tmp=*totCost/tmpgamma;
  tmpX=exp(-1*tmp);
  tmpgamma=(tmpB+*totCost*tmpX)/(((1+tmp)*tmpX)+tmpA-*rho*((1-pow(*beta, (tmpM+1)))/(1-*beta))); //lager ny tmpgamma basertpå gamle verdier
  tmp=*totCost/tmpgamma; // lager ny tmp utfra ny tmpgamma
  tmpX=exp(-1*tmp);  //lager ny tmpX utfra ny tmpgamma
  tmpA=*beta*(tmpA+(1+tmp)*tmpX);
  tmpB=*beta*(tmpB+ *totCost*tmpX);
  tmpM++;

  if(eliteAntTest-*totCost>-1e-13) // dette må gjøres pga unøyaktighet i utregningen
  {
    // regner ut eliteantverdier
    tmp=*totCost/(*gamma);
    *X=exp(-1*tmp);
    *gamma=(B+(*totCost)*(*X))/((1+tmp)*(*X)+A-*rho*((1-pow(*beta, (M+1)))/(1-*beta)));
    tmp=*totCost/(*gamma); // lager ny tmp
    *X=exp(-1*tmp); // lager ny X
    A=*beta*(A+(1+tmp)*(*X));
    B=*beta*(B+*totCost*(*X));
    M++;
    *eliteant=0;
  }
  // Hvis denne path ikke eksisterer legg til ny entry i både normal og elite
  if (vectorEntAcc<0)
  {
    _aaTable->addPVEntry(src_adr, tmpA, tmpB, tmpgamma, tmpM );
    _aaTable->addPVEntryElite(src_adr, tmpA, tmpB, tmpgamma, tmpM );
  }
  // oppdaterer eksisterende entry
  else
  {
    _aaTable->updatePathEntry(vectorEntAcc, tmpA, tmpB, tmpgamma, tmpM);
    if (*eliteant==0) //dette er eliteant og tabellen oppdateres
    {

      _aaTable->updatePathEntryElite(vectorEntAcc, A, B, *gamma, M);
    }
    else
    {
      //Hvis ikke eliteant legger inn gammaverdien  for alle maur slik at animering skal bli rett.
      *gamma=tmpgamma;
      *X=tmpX;
    }
  }
}
catch(...)
{
  feil=1;
  antallfeilCP++;
}
}





Packet * UpdatePathTable::generateUpdateAnt(Packet *p, int *_flag, double *X, int *totCost, double *gamma, double *beta)

{
  try
  {
    WritablePacket *q = p->clone()->uniqueify();
    int interface_nr=(PAINT_ANNO(p));
    click_ip *ip = q->ip_header();
    click_udp *udp = q->udp_header();

    // Set src og  dst ip-addr samt  port no
    in_addr tmp_src=p->ip_header()->ip_src;
    in_addr tmp_dst=p->ip_header()->ip_dst;

    ip->ip_src =_aaTable->getVlanIP(interface_nr);
    ip->ip_dst = tmp_src;
    ip->ip_id = random();
    ip->ip_ttl = 9;
    udp->uh_sport = p->udp_header()->uh_dport;
    udp->uh_dport = p->udp_header()->uh_sport;

    // Use route record to create source routing
    uint8_t *oa = q->network_header();
    int hlen = q->network_header_length();

    // Walk through options header
    for (int oi = sizeof(click_ip); oi < hlen; )
    {
      //	int pointer=oa[oi+2];
      unsigned type = oa[oi];
      if (type == IPOPT_EOL)
      {
        //   end of option list
        break;
      }
      else if (type == IPOPT_RR)
      {
        // Route Record
        oa[oi] = IPOPT_LSRR;
        int last_ptr = oa[oi+2]-5; // points to last route record
        int first_ptr = 3;

        if (last_ptr >= 3)
        {
          // 1 or more
          struct in_addr dest;
          memcpy(&dest, oa + oi + last_ptr, 4);   // Use last as destination
          ip->ip_dst = dest;
          memcpy(oa + oi + last_ptr, &p->ip_header()->ip_src, 4);   // Put source in last position
          last_ptr -= 4;
        }
        while (last_ptr > first_ptr)
        {
          // swap
          struct in_addr temp;
          memcpy(&temp, oa + oi + last_ptr, 4);
          memcpy(oa + oi + last_ptr, oa + oi + first_ptr, 4);
          memcpy(oa + oi + first_ptr, &temp, 4);

          first_ptr += 4;
          last_ptr -= 4;
        }
        oa[oi+2] = 4;
        break;
      }
    }
    // lager datalasten
    StringAccum udpload;
    udpload << *_flag; 	//explorer type
    udpload << "/";
    udpload << *X;	 	//exponentialfunksjon
    udpload << "/";
    udpload << *totCost;  // total cost
    udpload << "/";
    udpload << *gamma;     // gamma
    udpload << "/";
    udpload << *beta;       //beta
    udpload << "/";
    udpload << 0; // Brukes ikke til noe setter til 0
    udpload << "/";


    udpload << getCostVectorSize()-1; //antall hopp
    for (int i=0; i<getCostVectorSize()-1; i++)
    {

      udpload << "/";
      udpload << getCostVectorValue(i);

    }
    udpload << "!";  // Avslutter payload for å kunne skille virkelige data fra "søppel"
    String flag=udpload.data(); // datalasten
    int lengde = flag.length()- (q->length()-q->network_header_length()-sizeof(struct click_udp));

    Packet * wp= addPayload(q, flag); // legger inn datasaten
    return wp;

  }
  catch(...)
  {
    feil=1;
    antallfeilGUA++;
  }
}





Packet * UpdatePathTable::update_checksum2(WritablePacket *p)
{
  WritablePacket *wp = 0;
  if (!wp)
  {
    if (!(wp = p->uniqueify()))
      return 0;
    else
    {
      click_ip *iph = wp->ip_header();
      click_udp *udph = wp->udp_header();

      uint16_t len = ntohs(udph->uh_ulen);
      int hlen = p->network_header_length();

      iph->ip_sum = 0;
      iph->ip_sum = click_in_cksum(p->network_header(), hlen);

      udph->uh_sum = 0;
      unsigned csum = click_in_cksum((unsigned char *)udph, len);
      udph->uh_sum = click_in_cksum_pseudohdr(csum, iph, len);
    }
  }
  return (wp ? wp : p);
}

Packet * UpdatePathTable::addPayload(Packet *p, String payload)
{

  WritablePacket *wp = 0;
  if (!(wp = p->uniqueify()))
    return 0;
  // Add payload
  int udph_l = wp->network_header_length();
  wp->put(payload.length());
  memcpy(wp->data() + udph_l+8, payload.c_str(), payload.length());
  if (!wp)
    return 0;
  // Update IP and UDP length
  click_ip *iph = wp->ip_header();
  click_udp *udph = wp->udp_header();
  uint16_t len = ntohs(udph->uh_ulen)+payload.length()+9;
  udph->uh_ulen = htons(len);
  iph->ip_len = htons(p->length());

  return update_checksum2(wp);

}


void UpdatePathTable::push(int , Packet *p)
{
  out=0;
  feil=0;
  int explore=-1;
  double X=1;
  int totCost=1;
  double gamma=1;
  double tmpNAMtime=0;
  int eliteant=1;
  //Packet *pakke=handle_options(p);
  getPayload(p,  &explore, &rho, &beta, &totCost);
  calculatePayload(p, &X, &gamma, &beta, &rho, &tmpNAMtime, &eliteant, &totCost);
  Packet *pakke=generateUpdateAnt(handle_options(p), &eliteant, &X, &totCost, &gamma, &beta);

  StringAccum test86;
  test86 << "antallfeil i getPayload: ";
  test86 << antallfeilGP;
  test86 << " antallfeil i calculatePayload : ";
  test86 << antallfeilCP;
  test86 << " antallfeil i generateUpdateAnt :";
  test86 << antallfeilGUA;
  click_chatter (test86.data());

  if(feil==0)
  {
    output(out).push(pakke);
  }
  else
  {
    click_chatter( "UPDPATHTABLE VI HAR EN FEIL");
  }
}
#include <click/vector.cc>
CLICK_ENDDECLS
ELEMENT_REQUIRES(AntARPTable)
EXPORT_ELEMENT(UpdatePathTable)
