#! /usr/bin/perl -w

# make-ip-conf.pl -- make a Click IP router configuration
# Robert Morris, Eddie Kohler, David Scott Page
#
# Copyright (c) 1999-2000 Massachusetts Institute of Technology
# Copyright (c) 2002 International Computer Science Institute
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, subject to the conditions
# listed in the Click LICENSE file. These conditions include: you must
# preserve this copyright notice, and you cannot mention the copyright
# holders in advertising related to the Software without their permission.
# The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
# notice is a summary of the Click LICENSE file; the license in that file is
# legally binding.

# Make a Click IP router configuration.  This script generates a
# configuration using PollDevices. You can change it to use
# FromDevices; see the comment above the $ifs array, below.  The
# output is intended for the Linux kernel module; however, by making
# the change from PollDevices to FromDevices, and setting $local_host
# appropriately, the configuration will also work at userlevel.

# --------------------------- Begin configuration ----------------------------
#my $ifs = [ [ "vlan3", 3, 3, "10.1.5.1", "255.255.255.0", "BB:BB:BB:01:05:01" ],
#						[ "vlan4", 1, 2, "10.1.2.1", "255.255.255.0", "BB:BB:BB:01:02:01" ],
#						[ "vlan5", 5, 1, "10.1.3.1", "255.255.255.0", "BB:BB:BB:01:03:01" ],
#            [ "vlan6", 3, 0, "10.1.9.1", "255.255.255.0", "BB:BB:BB:01:09:01" ],
#           ];

# This used for testing purposes at MIT.
if ($#ARGV >= 0) {
  $ifs = [];
  for ($i = 0; $i < $ARGV[0]; $i++) {
    push @$ifs, [ "eth" . $i, 1, "1.0.0.2", "255.0.0.0", "00:00:c0:8a:67:ef" ];
  }
}

# Static routes to hosts/networks beyond adjacent networks specified in $ifs.
# One line per route, containing:
#   The destination address (host or network);
#   The mask;
#   The gateway IP address (next hop);
#   The output network interface name.
# A default route (mask 0.0.0.0) can be specified as the last entry.
#my $srts = [ [ "0.0.0.0", "0.0.0.0", "10.1.3.1", "vlan5" ]];
#my $srts = [];

# Set to, e.g., "Print(toh) -> Discard" for user-level.
my $local_host = "Discard";

# Set to 1 if you want the configuration to handle ICMP echo requests itself.
my $handle_pings = 1;

# --------------------------- End of configuration ---------------------------

my $nifs = $#$ifs + 1;
my $nsrts = $#$srts + 1;

print "// Auto-generated by make-ant-router.pl\n";

my $i;
my(@adr,@align,@anttable);
for($i = 0; $i < $nifs; $i++){
    printf("// %s %s %s\n",
           $ifs->[$i]->[0],
           $ifs->[$i]->[3],
           $ifs->[$i]->[5]);
    push @adr, sprintf(" %s %s",$ifs->[$i]->[0],$ifs->[$i]->[3]);
    push @align, sprintf("c%s 4 2",$i);
    push @anttable, sprintf("%s/%s %s %s", $ifs->[$i]->[0], $ifs->[$i]->[4], $ifs->[$i]->[2], $ifs->[$i]->[1]);
}


print "\nAddressInfo (\n\t", join(",\n\t", @adr), ");\n";

print "\nAlignmentInfo(\n\t", join(",\n\t", @align), ");\n";
                     	
print "\nat :: AntTable(\n\t", join(",\n\t", @anttable), ");\n";
print "// \{\}\n";
print <<"EOD;";

elementclass prepareIP1 {\$no, \$ip |
	/* Remove ethernet header, update source routing .. */
	//input -> Paint(\$no) -> Strip(14) -> MarkIPHeader() -> SourceRoute(\$ip) -> output;
	input -> Paint(\$no) -> Strip(14) -> MarkIPHeader() -> output;
};

elementclass prepareIP2 {\$no, \$ip |
	/* Drops link-level broadcast and multicast packets, update route record, time-to-live etc. */
	input -> DropBroadcasts
		-> cp0 :: PaintTee(\$no)
		-> gio0 :: IPGWOptions(\$ip)
		-> FixIPSrc(\$ip)
		-> dt0 :: DecIPTTL
		-> fr0 :: IPFragmenter(1500)
		-> atm :: AntTrafficMonitor(at, \$no, 0.85)
		-> output;
	dt0[1] -> ICMPError(\$ip, timeexceeded) -> [1] output;
	fr0[1] -> ICMPError(\$ip, unreachable, needfrag) -> [1] output;
	gio0[1] -> ICMPError(\$ip, parameterproblem) -> [1] output;
	cp0[1] -> ICMPError(\$ip, redirect, host) -> [1] output;
};

aat :: AntARPTable(vlan3, vlan4, vlan5, vlan6);
rt :: AntARP(aat);
ut::UpdateAATbl(aat);
utpath::UpdatePathTable(aat);	

sr :: SourceRoute(0.0.0.0) -> rt;	

ip :: CheckIPHeader(VERBOSE true, DETAILS true)-> DropBroadcasts
    -> ant :: IPClassifier(dst udp port 51234, src udp port 51234, -);

ant[0] -> ut;
ant[1] -> BackwardAnt(at) -> sr;
ant[2] -> sr;
ut[0]  -> fa :: ForwardAnt(at);
ut[1]  -> rt;
EOD;


# Link-level devices, classification, and ARP
print "\n// ARP responses are copied to each ARPQuerier and the host.\n";
printf("arpt :: Tee(%d);\n", $nifs + 1);
for($i = 0; $i < $nifs; $i++){
    my $devname = $ifs->[$i]->[0];
    my $ip = $ifs->[$i]->[3];
    my $ena = $ifs->[$i]->[5];
    print <<"EOD;";

// Input and output paths for $devname
c$i :: Classifier(12/0806 20/0001, 12/0806 20/0002, 12/0800, -);
FromDevice($devname) -> c$i;
out$i :: Queue(200) -> todevice$i :: ToDevice($devname);
//FromDevice($devname) -> ToDump(\"/bison/openwrt/logs/from_$ip\") -> c$i;
//out$i :: Queue(200) -> ToDump(\"/bison/openwrt/logs/to_$ip\") -> todevice$i :: ToDevice($devname);
arpq$i :: ARPQuerier($ip, $ena) -> out$i;
c$i\[0] -> ar$i :: ARPResponder(10.1.0.0/18 $ena) -> out$i;
c$i\[1] -> arpt;
c$i\[2] -> prepareIP1($i, $devname) -> ip;
c$i\[3] -> Discard;
arpt[$i] -> [1]arpq$i;
EOD;
}


# Local delivery path.
print "\n// Local delivery\n";
print "toh :: $local_host;\n";
print "arpt[$nifs] -> toh;\n";
print <<"EOD;";
rt[4] -> local :: IPReassembler -> ping_ipc :: IPClassifier(icmp type echo, -);
ping_ipc[0] -> ICMPPingResponder -> [0]rt;
ping_ipc[1] -> EtherEncap(0x0800, 1:1:1:1:1:1, 2:2:2:2:2:2) -> toh;
EOD;

# Forwarding path.
for($i = 0; $i < $nifs; $i++){
    my $i1 = $i;
    my $devname = $ifs->[$i]->[0];
    print <<"EOD;";

// Forwarding path for $devname
rt[$i1] -> p$devname :: prepareIP2($i, $devname);
p$devname\[0] -> [0]arpq$i;
p$devname\[1] -> rt;
EOD;
}

print "\n// Routing from ForwardAnt\n";
for($i = 0; $i < $nifs; $i++){
		my $i1 = $i + 1;
		my $devname = $ifs->[$i]->[0];
    my $ii = ip2i($ifs->[$i]->[3]);
    my $gw;
    if (($ii & 0xff) == 1) {
			$gw = i2ip(($ii & 0xffffff00) | 0x02);
		} else {
			$gw = i2ip(($ii & 0xffffff00) | 0x01);
		}
		print "fa[$i] -> p$devname;\n";
}
print "fa[$nifs] -> utpath;  // Packet is for this host\n";

my $last = $nifs + 1;
print "fa[$last] -> rt;\n";
print "utpath [0] -> rt;\n";
print "utpath [1] -> sr;\n";

sub ip2i {
    my($ip) = @_;
    my @a = split(/\./, $ip);
    my $i = ($a[0] << 24) + ($a[1] << 16) + ($a[2] << 8) + $a[3];
    return($i);
}
sub i2ip {
    my($i) = @_;
    my $a = ($i >> 24) & 0xff;
    my $b = ($i >> 16) & 0xff;
    my $c = ($i >> 8) & 0xff;
    my $d = $i & 0xff;
    return sprintf("%d.%d.%d.%d", $a, $b, $c, $d);
}
